
# All documentation (from which man page and help text is generated) is stored in the docstrins in each function.
# It consists of a number of fields of the following form:
#	"""FIELD: 'text
#	          .    All whitespace before the dot is ignored.'
#	"""

import re

def docs2dict(docstring):
	d = {}
	for match in re.finditer(r"(\w+): \s* ' ([^']*) '", docstring, re.X):
		field, value = match.groups()
		value = re.sub(r'\n[ \t]*([^. \t])', r' \1', value)
		value = re.sub(r'\n[ \t]*\.', r'\n', value)
		d[field] = value
	return d



import base64
import time
import urllib
import urllib2
import sys
import re

try:
    import simplejson as json
except ImportError:
    import json

from sslverify import ValidHTTPSHandler

from ovm_ctl import __version__ as VERSION

# This file contains the web api object itself, without all the fancy wrapping.
# If you want to use this for your python program, just create a apibindings object.
# This object contains all the calls - if you look at the calls list in __init__,
# 	the 2nd thing in each tuple is the python member name for that function.
# Example:
# >>> api = apibindings('example', 'passw0rd')
# >>> print api.disk_pool()
#
# IMPORTANT: Our actual API is not finalised yet,
# so if you use these be aware they may completely change some day with no warning.

class HTTPException(Exception):
	def __init__(self, message, retcode, response=None):
		Exception.__init__(self, message)
		self.retcode = retcode
		self.response = response

   
class MethodAndCredsRequest(urllib2.Request):
	def __init__(self, method, user, pswd, *args, **kwargs):
		self._method = method
		if 'headers' not in kwargs: kwargs['headers'] = {}
		kwargs['headers']['User-Agent'] = "OrionVM CLI bindings version %s" % VERSION
		if user and pswd:
			basicheader = "Basic " + base64.b64encode(user + ":" + pswd)
			kwargs['headers']['Authorization'] = basicheader
		urllib2.Request.__init__(self, *args, **kwargs)

	def get_method(self):
		return self._method if self._method else super(RequestWithMethod, self).get_method()


def push(url, post_data, user, pswd, verbose=False, type='GET', con=None, retcon=False):
	opener = con
	
	if verbose:
		print url, post_data

	data = None

	#string_s = StringIO.StringIO()
	#header_s = StringIO.StringIO()

	if post_data and type=='POST':
		data = urllib.urlencode(post_data)
	else:
		if post_data:
			url += '?' + '&'.join(['%s=%s' % (k,v) for k,v in post_data.items()])
		
	if not opener:		
		opener = urllib2.OpenerDirector()
		opener.add_handler(ValidHTTPSHandler())
		opener.add_handler(urllib2.HTTPDefaultErrorHandler())
		opener.add_handler(urllib2.HTTPErrorProcessor())

	req = MethodAndCredsRequest(type, user, pswd, url, data)
	try:
		response = opener.open(req)
	except urllib2.URLError, e:
		if not hasattr(e, 'code'):
			raise
		retcode = e.code
		page = e.read()
	else:
		retcode = 200
		page = response.read()
	
	if retcode != 200:
		raise HTTPException("API error %s" % retcode, retcode, response=page)

	if not retcon:
		return page

	return page, opener

def extra(isatty, api):
	if not isatty:
		print 'No moo\tDo have OVM power\tRun with -v'
		return 0
	enc = open('extra', 'r').read()
	enc = enc.split('\n')
	partial = [''] * len(enc[0])
	for x in range(len(enc[0])):
		for line in enc:
			partial[x] += line[x]
	dec = base64.decodestring('\n'.join(partial))
	print 'Sorry, no Super Cow Powers here.'
	time.sleep(1)
	print 'But we do have Super OrionVM Powers!'
	time.sleep(2)
	sys.stdout.write("You don't believe me")
	sys.stdout.flush()
	time.sleep(2)
	for msg in ['.', '.', '.', '?']:
		sys.stdout.write(msg)
		sys.stdout.flush()
		time.sleep(1)
	time.sleep(2)
	print dec
extra_r = [('', r'^(?:show|create|destroy|attach|detach) moo$', extra)]

class apibindings(object):
	def __init__(self, user, pswd):
		self.GET = 'GET'
		self.POST = 'POST'
		self.user = user
		self.pswd = pswd

		base = "https://panel.orionvm.com.au/api/"
		# list of (api url, binding name, method)
		calls = [(r'vm_pool', 'vm_pool', self.GET),
			(r'ip_pool', 'ip_pool', self.GET),
			(r'disk_pool', 'disk_pool', self.GET),
			(r'image_pool', 'image_pool', self.GET),
			(r'user_details', 'details', self.GET),
			(r'deploy_disk', 'deploydisk', self.POST),
			(r'create_disk', 'createdisk', self.POST),
			(r'allocate_ip', 'allocateip', self.POST),
			(r'vm_allocate', 'allocatevm', self.POST),
			(r'attach_disk', 'attachdisk', self.POST),
			(r'attach_ip', 'attachip', self.POST),
			(r'detach_disk', 'detachdisk', self.POST),
			(r'detach_ip', 'detachip', self.POST),
			(r'deploy', 'deploy', self.POST),
			(r'action', 'actionvm', self.POST),
			(r'drop_ip', 'dropip', self.POST),
			(r'drop_disk', 'dropdisk', self.POST),
			(r'drop_vm', 'dropvm', self.POST),
			(r'usage', 'usage', self.GET),
			(r'context', 'get_context', self.GET),
			(r'context', 'set_context', self.POST),
			(r'set_ram', 'set_ram', self.POST)]

		self.con = None
		for url, nme, type in calls:
			newfunc = self._docall(base + url, type)
			self.__dict__[nme] = newfunc


	def _docall(self, url, type):
		def fun(**kwargs):
			ret, self.con = push(url, kwargs, self.user, self.pswd, verbose=False, type=type, con=self.con, retcon=True)
			return json.loads(ret)

		return fun

# deprecated
def protectAPI(call, istty, *args, **kwargs):
	"""Wrap every api call with common error handling
	***DEPRECATED*** in favour of more general error handling higher up in the call stack."""
	try:
		return call(*args, **kwargs)
	except HTTPException, e:
		if istty:
			print 'Error while calling API function %s: Returned code %d' % (call.__name__, e.retcode)
			if e.response:
				if raw_input('View entire web response? (y/n) > ') == 'y':
					print e.response
		raise



import show
import create
import attach
import detach
import boot
import shutdown
import console
import destroy
import context
import wizard
import password
import setram
import sys

# This file contains the regex patterns that define our commands.
# It's a weird way to do things but it's worked out quite nice.

# route: (fixed string for pre-matching, regex, function)

wizardroutes = [('wizard vm', r'^wizard vm ([^ ]+)(?: with(?: ram ([^ ]+))?(?: disk ([^ ]+))?(?: image ([^ ]+))?)?( and boot)?$', wizard.vmwizard)]

showroutes = [('show ips', r'^show ips$', show.showips),\
              ('show ip', r'show ip ([^ ]+)$', show.showip),\
              ('show vms', r'^show vms$', show.showvms),\
              ('show vm', r'^show vm ([^ ]+)$', show.showvm),\
              ('show disks', r'^show disks$', show.showdisks),\
              ('show disk', r'^show disk ([^ ]+)$', show.showdisk),\
              ('show usage', r'^show usage$', show.showuse),\
              ('show balance', r'^show (?:balance|me the money)$', show.showbalance),\
              ('show images', r'^show images$', show.showimages),\
              ('show context', r'^show context key ([^ ]+) on vm ([^ ]+)$', show.showcontext)]

createroutes = [('create disk', r'^create disk ([^ ]+) with size ([^ ]+)(?: and image ([^ ]+))?$', create.createdisk), \
                ('create ip', r'^create ip with friendly ([^ ]+)$', create.createip),\
                ('create vm', r'^create vm ([^ ]+) with ram ([0-9]+[MG])(?: of type ([^ ]+))?$', create.createvm),\
                ('clone disk', r'^clone disk ([^ ]+) from ([^ ]+)(?: with size ([^ ]+))?$', create.clonedisk)]


attachroutes = [('attach disk', r'^attach disk ([^ ]+) to ([^ ]+) with target ([^ ]+)$', attach.attachdisk),\
                ('attach ip', r'^attach ip ([^ ]+) to vm ([^ ]+)$', attach.attachip)]

detachroutes = [('detach disk', r'^detach disk ([^ ]+) from ([^ ]+)$', detach.detachdisk),\
                ('detach ip', r'^detach ip ([^ ]+) from ([^ ]+)$', detach.detachip)]

bootroutes = [('boot vm', r'^boot vm ([^ ]+)$', boot.bootvm)]

shutdownroutes = [('shutdown vm', r'^shutdown vm ([^ ]+)$', shutdown.shutdownvm)]

consoleroutes = [('console', r'^console ([^ ]+)$', console.runconsole)]

destroyroutes = [('destroy ip', r'^destroy ip ([^ ]+)$', destroy.destroyip),\
                ('destroy disk', r'destroy disk ([^ ]+)$', destroy.destroydisk),\
                ('destroy vm', r'destroy vm ([^ ]+)$', destroy.destroyvm)]

contextroutes = [('add context', r'^add context to vm ([^ ]+) with ([^ ]+) = (.*)$', context.addcontext),
                 ('clear context', r'^clear context key ([^ ]+) from vm ([^ ]+)$', context.clearcontext)]

passwordroutes = [('set password', r'set password on vm ([^ ]+)(?: to (.+))?', password.setpass)]

def comment(api,istty):
	"""call: '# COMMENT'
	description: 'A line beginning with a "#" is a comment line. This line will be ignored.
	             .This can be useful when writing scripts to be used with the -f option.'
	"""
	return 0

def quit(*args):
    """call: 'quit'
    description: 'Exit the program. Has no effect if given in command line options.
                 .Note that an EOF (Ctrl-D) can also be used to exit.'
    """
    sys.exit(0)

miscroutes = [('#', r'^(?:[ \t]*#.*)?$', comment),\
              ('quit', r'quit', quit),
              ('set ram', r'set ram on vm ([^ ]+) to ([^ ]+)$', setram.setram)]

routes = sum([
	wizardroutes,
	showroutes,
	createroutes,
	attachroutes,
	detachroutes,
	bootroutes,
	shutdownroutes,
	consoleroutes,
	destroyroutes,
	contextroutes,
	passwordroutes,
	miscroutes]
	,[])



from show import getvmby
from webbindings import HTTPException
import re

def setram(vmname, ram, istty, api):
	"""call: 'set ram on vm NAME to SIZE'
	description: 'Update the ram of a VM. VM Must be stopped.'
	args: 'NAME: The name of the VM.
	      .SIZE: Amount of RAM to set on the VM.
	      .      Format is a number followed by M for megabytes or G for gigabytes.
	      .		 Value must be between 512M and 64G (65536M)
	errors: 'VM name is invalid or VM does not exist: Return exit code 3
	        .Incorrect format for SIZE: Return exit code 5'
	"""
	vm = getvmby(vmname, api, what='hostname')
	if not vm:
		if istty:
			print 'VM %s does not exist' % vmname
		return 3

	if vm['state'] != 0:
		if istty:
			print vmname, " is not in a state to have ram changed"
		return 4

	ram = ram.upper()
	if not re.match('^(((\d*[.])?\d+G)|([1-9][0-9]+M))$', ram):
		if istty:
			print 'Bad format for ram size: Must be a number followed by "M" for megabytes or "G" for gigabytes.'
		return 5

	unit = ram[-1]
	ram = float(ram[0:-1])
	if unit == "G":
		ram = int(ram * 1024)
		
	if ram < 512 or ram > 65536:
		if istty:
			print 'RAM must be between 512M and 64G (65536M)'
		return 5	
		
	ram = "%sM" % ram

	api.set_ram(vmid=vm['vm_id'], ram=ram)


#!/usr/bin/env python

DEBUG=False # Change this to true to have --debug by default. Useful if code is failing before option parsing is reached.

import sys
import re
import os
import getpass
import sys
from getopt import gnu_getopt, GetoptError
from ovm_ctl import routes
from ovm_ctl.webbindings import apibindings, extra_r, HTTPException
from ovm_ctl.helping import helping
from ovm_ctl.parsedocs import docs2dict

def quit(*args):
	"""call: 'quit'
	description: 'Exit the program. Has no effect if given in command line options.
	             .Note that an EOF (Ctrl-D) or blank line can also be used to exit.'
	"""
	sys.exit(0)

def notimp(*args):
	print "Not implemented"

routes = routes.routes + \
 	[('help', r'^help(?: (.*))?$', helping)]

croutes = [(re.compile(pattern), fun) for padding, pattern, fun in routes + extra_r]

MIN_CLOSENESS = 4 # Min leading characters to match before deciding to print a "did you mean"

def cmd_parse(api, cmd, istty):
	for cpattern, fun in croutes:
		match = cpattern.match(cmd.strip())
		if not match:
			continue
		groups = match.groups()
		groups = list(groups) + [istty, api]
		try:
			ret = fun(*groups)
			if ret == None: # No return -> success
				ret = 0
			return ret
		except Exception, ex:
			if istty:
				print """Unknown error. Command failed.
Error message: "%s"
Maybe the web API is inaccessible, or your credentials are wrong.
If you are trying to do something to a disk or IP, you may have to detach it first.
Most operations on a vm can only be done while it is shut down.""" % str(ex)
			if DEBUG:
				raise
			return 2

	if istty:
		did_you_mean(cmd, routes)
	return 1

def did_you_mean(cmd, routes):
	"""Print a message along the lines of "did you mean: blah" or "blah not formatted correctly",
	by trying to match the beginning of cmd to the fixed beginnings of each command"""
	best = []
	bestlength = 0
	for fixedstr, pattern, func in routes:
		for l in range(len(fixedstr))[::-1]:
			if cmd.startswith(fixedstr[:l+1]):
				if l+1 > bestlength:
					best = [(fixedstr, func.__doc__)]
					bestlength = l+1
				elif l+1 == bestlength:
					best += [(fixedstr, func.__doc__)]
				break
	for x in best:
		if bestlength == len(x[0]): # if prefix matches the whole fixedstr
			print 'Command incorrectly formatted'
			print 'Basic usage: %s' % docs2dict(x[1])['call']
			print 'For more information, try "help %s"' % x[0]
			return
	print 'Command not found.'
	if bestlength >= MIN_CLOSENESS:
		print 'Did you mean:'
		for x in best:
			print docs2dict(x[1])['call']

def repl(api, istty):
	if istty:
		try:
			print "Hi %(firstname)s %(lastname)s!" % api.details()
		except Exception:
			pass
		print "Welcome to the OrionVM command line bindings. For more information please type help."
		print "For full documentation exit the program and run 'man ovm-ctl'."
	while 1:
		try:
			cmd = raw_input(istty and ">> " or "")
		except EOFError:
			return 0

		ret = cmd_parse(api, cmd, istty)
		if not istty and ret:
			return ret

def get_creds(isatty, credfile, user=None, pswd=None):
	from_file = False
	user_from_file = False
	pswd_from_file = False
	asksave = False
	if not user:
		if os.path.exists(credfile):
			creds = open(credfile, 'rU').read()
			creds = creds.split("\n")
			if len(creds) > 0 and creds[0] != '':
				user = creds[0]
				user_from_file = user != ''
			if len(creds) > 1 and creds[1] != '':
				pswd = creds[1]
				pswd_from_file = pswd != ''
			from_file = user_from_file and pswd_from_file

	if not user:
		user = raw_input("Username: ")
		asksave = True

	if not pswd:
		if isatty and user_from_file:
			print "Username: %s" % user
		pswd = getpass.getpass("Password: ")

	# Only ask to save if default credfile doesn't exist, no other credfile given, user is not blank.
	if user and isatty and asksave:
		while True:
			response = raw_input("Would you like to save your username as the default? [yes/no/never] > ")
			if response == 'yes':
				credfd = open(credfile, 'w')
				if os.name == 'posix':
					os.chmod(credfile, 0600)
				credfd.write(user+'\n')
				while True:
					response = raw_input("Also save your password (this may not be secure)? [yes/no] > ")
					if response not in ('yes', 'no'):
						continue
					if response == 'yes':
						credfd.write(pswd+'\n')
					credfd.close()
					break
				break
			elif response == 'never':
				open(credfile, 'w') # Touch to create blank file
				if os.name == 'posix':
					os.chmod(credfile, 0600)
				break
			elif response == 'no':
				break

	return (user, pswd, from_file)

def creds(isatty, user=None, pswd=None, credfile=None):
	if credfile == None:
		credfile = os.path.expanduser("~/.orionauth")
	
	while 1:
		user, pswd, from_file = get_creds(isatty, credfile, user=user, pswd=pswd)
		api = apibindings(user, pswd)
		try:
			api.details()
			return (user, pswd)
		
		except HTTPException, e:
			if e.retcode == 401:
				if not isatty:
					raise

				user = None
				pswd = None	

				if from_file:
					try:
						os.remove(credfile)
					except OSError:
						raise Exception("Invalid userame and/or password in auth file %s" % credfile)
				else:
					print "Invalid username and/or password"
			
			else:
				raise

# Dict of distinct options, where each entry is identifier:([short names], [long names], argname, docstring)
# with argname = '' for no args
options = {
	'user': (['u'], ['user', 'username', 'auth'], 'user[:pass]', "If given, specifies an OrionVM account to log in as. Optionally, the password can also be given, seperated from the username by a colon. This option overrides anything read from an authfile."),
	'authfile': (['a'], ['authfile'], 'filename', "Use the given file as the authfile (see Authentication) instead of the default."),
	'file': (['f'], ['file'], 'filename', "Read a list of commands from a file, instead of stdin. Implies -q unless overriden with -v. Note that options are processed in the order they are given and any later option overrides an earlier one."),
	'verbose': (['v'], ['verbose'], '', "Force verbose mode even when stdin or stdout is not a tty. Causes the program to generate human-readable outputs."),
	'quiet': (['q'], ['quiet'], '', "Force quiet mode even when stdin and stdout are a tty. Causes the program to generate little or no output."),
	'debug': ([], ['debug'], '', "Report the underlying exceptions in the python code. This option is intended only for debugging a problem after editing the code.") # Debug mode (don't catch errors)
}

def getopt_values():
	"""Return (short_opts, long_opts):
		short_opts: a string for shortopts suitable for getopts
		long_opts: a list for longopts also suitable for getopts"""
	short_opts = ''
	long_opts = []
	for shorts, longs, needs_arg, doc in options.values():
		for short_opt in shorts:
			short_opts += short_opt
			if needs_arg:
				short_opts += ':'
		if needs_arg:
			longs = [opt + '=' for opt in longs]
		long_opts += longs
	return (short_opts, long_opts)

def match_opt(opt, opt_name):
	"""Returns true if opt is one of the options listed under opt_name in options"""
	opt = opt[1:] # opt may be '-x' or '--abc', now is 'x' or '-abc'
	if opt in options[opt_name][0]:
		return True
	opt = opt[1:] # if short, is now ''. if long, is now 'abc'
	return opt in options[opt_name][1]

def main(args, istty):
	short_opts, long_opts = getopt_values()
	authfile = None
	user = None
	pswd = None

	try:
		opts, args = gnu_getopt(args, short_opts, long_opts)
	except GetoptError, ex:
		if istty:
			print 'Error parsing command line arguments: %s' % str(ex)
		return 1

	read_from_file = None
	for opt, value in opts:
		if match_opt(opt, 'verbose'):
			istty = True
		elif match_opt(opt, 'quiet'):
			istty = False
		elif match_opt(opt, 'user'):
			if ':' in value:
				user, pswd = value.split(':')
			else:
				user = value
		elif match_opt(opt, 'authfile'):
			authfile = value
		elif match_opt(opt, 'debug'):
			global DEBUG
			DEBUG = True
		elif match_opt(opt, 'file'):
			try:
				if value.startswith("./"):
					value = value[2:]
				read_from_file = open(value, 'rU')
			except IOError:
				print 'Could not open file "%s" for reading.' % value
				raise
			istty = False
		else:
			if istty:
				print 'Warning: option %s not implemented' % opt

	user, pswd = creds(istty, user, pswd, authfile)

	if read_from_file:
		sys.stdin = read_from_file

	api = apibindings(user, pswd)
	if args != []: # got cmdline args
		cmd = " ".join(args)
		ret = cmd_parse(api, cmd, istty)
		return ret
	else:
		return repl(api, istty)

if __name__ == "__main__":
	istty = sys.stdout.isatty() and sys.stdin.isatty()
	try:
		sys.exit(main(sys.argv[1:], istty))
	except SystemExit:
		raise
	except BaseException, ex:
		if DEBUG:
			if isinstance(ex, HTTPException):
				print 'DEBUG: Recieved response from web server:'
				print ex.response
			raise
		if istty:
			print 'An error occurred. Perhaps the api is inaccessible?'
			print ex
		sys.exit(2)



#!/usr/bin/env python
import urllib2
import httplib
import ssl
import socket
import os
import re

CA_FILE = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'ca.pem')

class CertificateError(Exception):
    pass

# The following 2 functions are from Python 3 lib/ssl
# To bring SSL hostname matching to Python 2
def ssl_dnsname_match(dn, hostname, max_wildcards=1):
	"""Matching according to RFC 6125, section 6.4.3

	http://tools.ietf.org/html/rfc6125#section-6.4.3
	"""
	pats = []
	if not dn:
		return False

	parts = dn.split(r'.')
	leftmost = parts[0]
	remainder = parts[1:]

	wildcards = leftmost.count('*')
	if wildcards > max_wildcards:
		# Issue #17980: avoid denials of service by refusing more
		# than one wildcard per fragment.  A survery of established
		# policy among SSL implementations showed it to be a
		# reasonable choice.
		raise CertificateError(
			"too many wildcards in certificate DNS name: " + repr(dn))

	# speed up common case w/o wildcards
	if not wildcards:
		return dn.lower() == hostname.lower()

	# RFC 6125, section 6.4.3, subitem 1.
	# The client SHOULD NOT attempt to match a presented identifier in which
	# the wildcard character comprises a label other than the left-most label.
	if leftmost == '*':
		# When '*' is a fragment by itself, it matches a non-empty dotless
		# fragment.
		pats.append('[^.]+')
	elif leftmost.startswith('xn--') or hostname.startswith('xn--'):
		# RFC 6125, section 6.4.3, subitem 3.
		# The client SHOULD NOT attempt to match a presented identifier
		# where the wildcard character is embedded within an A-label or
		# U-label of an internationalized domain name.
		pats.append(re.escape(leftmost))
	else:
		# Otherwise, '*' matches any dotless string, e.g. www*
		pats.append(re.escape(leftmost).replace(r'\*', '[^.]*'))

	# add the remaining fragments, ignore any wildcards
	for frag in remainder:
		pats.append(re.escape(frag))

	pat = re.compile(r'\A' + r'\.'.join(pats) + r'\Z', re.IGNORECASE)
	return pat.match(hostname)

def ssl_match_hostname(cert, hostname):
	"""Verify that *cert* (in decoded format as returned by
	SSLSocket.getpeercert()) matches the *hostname*.  RFC 2818 and RFC 6125
	rules are followed, but IP addresses are not accepted for *hostname*.

	CertificateError is raised on failure. On success, the function
	returns nothing.
	"""
	if not cert:
		raise ValueError("empty or no certificate")
	dnsnames = []
	san = cert.get('subjectAltName', ())
	for key, value in san:
		if key == 'DNS':
			if ssl_dnsname_match(value, hostname):
				return
			dnsnames.append(value)
	if not dnsnames:
		# The subject is only checked when there is no dNSName entry
		# in subjectAltName
		for sub in cert.get('subject', ()):
			for key, value in sub:
				# XXX according to RFC 2818, the most specific Common Name
				# must be used.
				if key == 'commonName':
					if ssl_dnsname_match(value, hostname):
						return
					dnsnames.append(value)
	if len(dnsnames) > 1:
		raise CertificateError("hostname %r "
			"doesn't match either of %s"
			% (hostname, ', '.join(map(repr, dnsnames))))
	elif len(dnsnames) == 1:
		raise CertificateError("hostname %r "
			"doesn't match %r"
			% (hostname, dnsnames[0]))
	else:
		raise CertificateError("no appropriate commonName or "
			"subjectAltName fields were found")

# http://stackoverflow.com/questions/6648952/urllib-and-validation-of-server-certificate
class ValidHTTPSConnection(httplib.HTTPConnection):
	"This class allows communication via SSL."

	default_port = httplib.HTTPS_PORT

	def __init__(self, *args, **kwargs):
		httplib.HTTPConnection.__init__(self, *args, **kwargs)

	def connect(self):
		"Connect to a host on a given (SSL) port."
		
		self.sock = socket.create_connection( (self.host, self.port),
		    self.timeout, self.source_address)
            
		if self._tunnel_host:
			self._tunnel()
			
		global CA_FILE
		self.sock = ssl.wrap_socket(self.sock,
			ca_certs=CA_FILE, cert_reqs=ssl.CERT_REQUIRED)
	
		cert = self.sock.getpeercert()
		ssl_match_hostname(cert, self.host)


class ValidHTTPSHandler(urllib2.HTTPSHandler):

	def https_open(self, req):
			return self.do_open(ValidHTTPSConnection, req)



"""Functions for formatting man pages"""
import time
import re

# This stuff is kinda nasty and specifically for writing man pages.
# Use it if you want but ovm-ctl itself never runs this file.

def title(name, section, date=None, source='', manual=''):
	"""Default date is current date. Set date to '' for no date. Should be YYYY-MM-DD
	source should be the origin of the command/function/subject.
	manual should be the title of the group of man pages (eg. 'Linux Programmers Manual')"""
	if date == None:
		year, month, day = time.localtime()[:3]
		date = '%04d-%02d-%02d' % (year, month, day)
	return '.TH %s %d %s %s %s\n' % (name, section, date, source, manual)

def heading(name, level=0):
	"""level between 0 and 2, 0 is more major"""
	if level == 0:
		request = 'SH '
	elif level == 1:
		request = 'SS '
	elif level == 2:
		request = 'TP\n'
	else:
		raise ValueError()
	return '.%s%s\n' % (request, name)

def multiheading(names):
	"""Makes a heading of level 2 with multiple lines (eg. a list of aliases).
	names is a non-empty list of strings."""
	if not names:
		raise ValueError()
	s = '.TP\n%s\n' % names[0]
	s += ''.join(['.TQ %s\n' % x for x in names[1:]])
	return s

def parbreak():
	"""Paragraph break"""
	return '.P\n'

def indent():
	return '.RS\n'

def deindent():
	return '.RE\n'

def itemize(bullet, items):
	"""As the LaTeX macro. Starts each item in items with bullet"""
	s = ''
	s += indent()
	for item in items:
		s += heading(bullet, 2) + item + '\n'
	s += deindent()
	return s

def code(text):
	"""Formats the text as code (fixed width, etc)"""
	return '.EX\n%s\n.EE\n' % text

def formatting(BorI, text, prefix='', suffix=''):
	"""BorI: 'B' for bold or 'I' for italic
	Concats prefix, text, suffix and makes text bold or italic"""
	if prefix:
		return '.R%s "%s" "%s" "%s"\n' % (BorI, prefix, text, suffix)
	if suffix:
		return '.%sR "%s" "%s"\n' % (BorI, text, suffix)
	return '.%s "%s"\n' % (BorI, text)

def bold(text, prefix='', suffix=''):
	return formatting('B', text, prefix, suffix)

def italic(text, prefix='', suffix=''):
	return formatting('I', text, prefix, suffix)

def begin_synopsis(name):
	return '.SY %s\n' % name

def option(opt, arg=''):
	"""Describes an optional argument in a command synopsis"""
	s = '.OP %s' % opt
	if arg:
		s += ' ' + arg
	s += '\n'
	return s

def end_synopsis():
	return '.YS\n'

def gen_synopsis(name, options=[], args=[], optional_args=[]):
	"""Generates a synopsis from options. Also puts args at the end.
	options: A list of ([shorts], [longs], arg, docstring) where:
		Each item is a distinct option - all options within one item are aliases
		arg is '' if no arg is needed, or else is the string that describes the arg.
		eg: (['f'], ['file'], 'filename') for a option -f or --file that takes an arg for a filename.
	args, optional_args is a list of argument descriptors"""
	result = begin_synopsis(name)
	shorts = []
	for s, l, a, doc in options: # Group non-arg shorts
		if not a:
			shorts += s
	result += option('-' + ''.join(shorts))
	for s, l, a, doc in options:
		aliases = []
		if a:
			aliases += ['-'+x for x in s]
		aliases += ['--'+x for x in l]
		if len(aliases) > 1:
			opt = '(%s)' % '|'.join(aliases)
		else:
			opt = aliases[0]
		result += option(opt, a)
	result += ''.join(['.I %s\n' % x for x in args])
	result += ''.join(['.RI [ %s ]\n' % x for x in optional_args])
	result += end_synopsis()
	return result

def check_result(s):
	"""Double-check a finished man page for blank lines (which break the formatting)
	Outputs a fixed copy where blank lines are now "blank requests" == "." """
	ret = ''
	for x in s.split('\n'):
		if not x:
			x = '.'
		ret += x + '\n'
	return ret

def fix_text(s):
	"""Takes a block of text and puts in newlines at the end of sentences. Also escapes backslashes."""
	s = escape_text(s)
	s = re.sub(r'\.(\s|$)', '.\n', s) # Put in newlines where needed
	s = re.sub(r'(^|\n)[ \t]+', r'\1', s) # Take out leading whitespace
	if s[-1] != '\n':
		s += '\n'
	return s

def escape_text(s):
	"""Escape any backslashes in text."""
	return s.replace('\\', '\\\\')



from show import getvmby
import re
from webbindings import HTTPException

def attachdisk(diskname, vmname, target, istty, api):
	r"""call: 'attach disk DISK to VM with target TARGET'
	    description: 'Attach a given disk to a given VM, locking the disk
	                 .and making it appear as a device on the VM.'
	    args: 'DISK: The name of the disk to attach.
	          .VM: The name of the VM to attach to.
	          .TARGET: The name of the device (the device will appear as /dev/TARGET).
	          .        For a linux vm:
	          .            Must be of form: "xvd%s%d" ==> ((a, b, c, ..., aa, ab, etc), positive integer),
	          .            eg: "xvda1", "xvda2", "xvdb1", "xvdb25".
	          .            Note: "xvdz" is reserved. Valid range is "xvda"-"xvdy", then "xvdaa", etc.
	          .        For a windows vm:
	          .            Must be of form: "hd%s" ==> (a, b, c, ..., aa, ab, ...)
	          .            eg. "hda", "hdb", "hdog", "hdzzz"'
	    errors: 'For non-existing or invalid (eg. running) VM: Returns exit code 3
	            .For non-existing or invalid Disk: Returns exit code 4
	            .For invalid target: Returns exit code 5'
    """
	print diskname, vmname, target
	vm = getvmby(vmname, api, what='hostname')
	if vm is None:
		if istty:
			print "VM %s does not exist" % vmname
		return 3
	if vm['state'] != 0:
		if istty:
			print "VM %s is not currently shut down" % vmname
		return 3

	if not diskname in [disk['name'] for disk in api.disk_pool()]:
		if istty:
			print "Disk %s does not exist" % diskname
		return 4

	# Note xvdz is reserved for context store access / future use
	if (target.startswith('xvdz') or not re.match('xvd[a-z]+[1-9][0-9]*', target)) and not re.match('hd[a-z]+', target):
		if istty:
			print "Target %s is not valid" % target
		return 5

	vmid = vm['vm_id']
	api.attachdisk(vmid=vmid, diskname=diskname, target=target, readonly=False)
	if istty:
		print "Attached %s to %s" % (diskname, vmname)

def attachip(ipname, vmname, istty, api):
        r"""call: 'attach ip IP to vm VM'
        description: 'Attach a given IP to a given VM,
	             .creating an ethernet interface on the VM.'
        args: 'IP: The IP Address or Friendly Name of the IP to attach.
	      .VM: The name of the VM to attach it to.'
        errors: 'For non-existing or invalid VM: Return exit code 3
	        .For non-existing or invalid IP: Return exit code 4'
        """
	vm = getvmby(vmname, api, what='hostname')
	if vm is None:
		if istty:
			print "VM %s does not exist" % vmname
		return 3

	ipaddr = None
	ip_pool = api.ip_pool()
	for ip in ip_pool:
		if ip['ip'] == ipname:
			ipaddr = ip['ip']
	if not ipaddr:
		for ip in ip_pool:
			if ip['friendly'] == ipname:
				ipaddr = ip['ip']
	if not ipaddr:
		if istty:
			print "IP %s does not exist" % ipname
		return 4
	
	vmid = vm['vm_id']
	api.attachip(ip=ipaddr, vmid=vmid)

	if istty:
		print "Attached ip %s to %s" % (ipname, vmname)



import show, attach, create
from boot import bootvm
from password import setpass
from context import addcontext
from time import sleep

def vmwizard(name, memsize, disksize, image, boot, isatty, api):
	"""call: 'wizard vm NAME [with [ram MEMSIZE] [disk DISKSIZE] [image IMAGE]] [and boot]'
	description: 'This command is a wizard to create a ready-to-run server with the given NAME.
	             .Any arguments not provided will be prompted. If the command has "and boot" on the end,
	             .the machine will be booted immediately after creation.
	             .Further details:
	             .    Create a new VM and call it NAME. Create a disk and ip (of the same NAME) and attach them.
	             .    The disk will have size DISKSIZE and image IMAGE. The VM will have ram MEMSIZE.
	             .    If "and boot" is given, the VM is then booted.
	             .    If "and boot" is not given, it is only prompted if no other arguments are provided.'
	args: 'NAME: The name to give to the VM, disk and ip address to be created.
	      .MEMSIZE: The amount of RAM to give the VM.
	      .         Format is a number followed by M for megabytes or G for gigabytes.
	      .DISKSIZE: The size of the new disk. See the SIZE argument for "create disk".'
	output: 'If creating a windows vm, prints the inital password'
	errors: 'NOTE: In the event of any sort of error, the VM may have only been partially constructed.
	        .      Please use the show commands and check what has and has not been created.
	        .Not all arguments provided and in quiet mode: Return exit code 1 (invalid command)
	        .Incorrect format for MEMSIZE: Return exit code 3
	        .Incorrect format for DISKSIZE: Return exit code 4
	        .Invalid value for IMAGE: Return exit code 5
	        .Already a disk, ip or VM called NAME: Return exit code 6
	        .An error occurred in booting the VM: Return exit code 7
	        .    Note: This means that the VM was successfully created.
	        .    Failure to boot means it is now in a Shut Down state.
	        .Any other error occurred during the operation: Return exit code 8
"""
	boot = boot == " and boot"

	# Get inputs
	if not isatty and not (memsize and disksize and image):
		return 1
	askboot = not (memsize or disksize or image or boot)
	if not memsize:
		memsize = raw_input("How much RAM? ")
	if not disksize:
		disksize = raw_input("How much disk space? ")
	if not image:
		show.showimages(isatty, api)
		image = raw_input("Please select one of the images above: ")
	if askboot:
		boot = raw_input("Would you like to boot the VM immediately? (y/n) > ") == "y"

	if show.getvmby(name, api, what='hostname'):
		if isatty:
			print "A VM with that name already exists."
		return 6

	type = ['paravirt', 'HVM']['windows' in image]
	ret = create.createvm(name, memsize, type, isatty, api)
	if ret == 3:
		return 3
	if ret:
		raise Exception("Unknown return value from createvm: %d"%ret)

	ret = create.createdisk(name, disksize, image, isatty, api)
	if ret == 3:
		return 5
	if ret == 4:
		return 4
	if ret == 5:
		return 6
	if ret == 6:
		return 8
	if ret:
		raise Exception("Unknown return value from createdisk: %d"%ret)

	ret = attach.attachdisk(name, name, ('windows' in image) and "hda" or "xvda1", isatty, api)
	if ret:
		raise Exception("Unexpected return value from attachdisk: %d"%ret)

	ret = create.createip(name, isatty, api)
	if ret:
		raise Exception("Unknown return value from createip: %d"%ret)

	ret = attach.attachip(name, name, isatty, api)
	if ret:
		raise Exception("Unknown return value from attachip: %d"%ret)

	if 'windows' in image:
		addcontext(name, 'WINDOWS', 'true', isatty, api)
		setpass(name, '', isatty, api)

	if boot:
		ret = bootvm(name, isatty, api)
		if ret == 5:
			return 7
		if ret == 6:
			return 8
		if ret:
			raise Exception("Unexpected return value from bootvm: %d"%ret)



import time, sys
from show import getvmby, get_ip


def destroyip(ipname, istty, api):
	"""call: 'destroy ip IP'
	description: 'Unallocate the given IP, which must not be locked (in use by a VM).
	             .Once an IP has been unallocated, there is no guarentee
	             .you can get the same one back again.
	             .Note that unlocked IPs still incur a charge until they have been destroyed.'
	args: 'IP: The IP address or friendly name to unallocate.'
	errors: 'IP is invalid, or no matching friendly name found: Returns exit code 3'
	"""
	ip = get_ip(ipname, api, istty)
	if not ip:
		if istty:
			print '%s not a valid address and no matching friendly name found' % ipname
		return 3

	api.dropip(ip=ip)

	if istty:
		print "Destroyed ip %s" % ipname
	

def destroydisk(diskname, istty, api):
	"""call: 'destroy disk DISK'
	description: 'Destroys the given disk that is not locked (attached to a VM).
	             .All data on the disk is lost. OrionVM is not responsible for any data lost this way.
	             .Please be very careful when dealing with unlocked disks.'
	args: 'DISK: Name of the disk to destroy.'
	errors: 'An error occurs while waiting for disk to be destroyed: Returns exit code 3
	        .    NOTE: When this occurs, the disk may or may not be destroyed.
	        .    Please run "show disks" and check if the disk is on the list.'
	"""

	api.dropdisk(diskname=diskname)

	time.sleep(1)

	if istty:
		sys.stdout.write("Dropping disk...")
		sys.stdout.flush()
	try:
		diskexists = True
		while diskexists:
			diskexists = False

			disks = api.disk_pool()
			for disk in disks:
				if disk['name'] == diskname:
					diskexists = True
					time.sleep(1)
					if istty:
						sys.stdout.write('.')
						sys.stdout.flush()
					continue
	except:
		if istty:
			print "An error occurred while dropping disk"
			print 'Please try "show disks" to check if your disk was successfully dropped'
		return 3

	if istty:
		print "Done"

def destroyvm(vmname, istty, api):
	"""call: 'destroy vm VM'
	description: 'Destroy given non-running instance.
	             .Note that any attached disks or IPs will still exist and continue to incur charges.'
	args: 'VM: Name of the instance to delete.'
	errors: 'VM does not exist: Return exit code 3'
	"""
	vm = getvmby(vmname, api, what='hostname')	
	if vm is None:
		print "VM does not exist"
		return 3

	vmid = vm['vm_id']

	api.dropvm(vmid=vmid)

	if istty:
		print "Destroyed vm %s" % vmname



import time
import sys
import re

def createdisk(diskname, size, image, isatty, api):
	""" call: 'create disk NAME with size SIZE [and image IMAGE]'
	description: 'Create a new storage volume and load it with the given software distribution.'
	args: 	'NAME: The name to give to the new disk.
	        .SIZE: The size of the new disk. Can be given in MB, GB or TB.
	        .      For example: "200G", "20g", "1.5T", "2048m". Value is rounded down to nearest GB.
	        .IMAGE: The software distribution to put onto the new disk. For a list of valid image names,
	        .        see the "show images" command. If not provided, creates a blank disk.'
	errors:	'If the given image is not valid, exit code 3 is returned.
	        .If the size is not valid, exit code 4 is returned.
	        .If there already exists a disk with name NAME, exit code 5 is returned.
	        .If an error occurs before creation is complete, exit code 6 is returned.
	        .    NOTE: The creation continues in the background, you must "show disks" to check
	        .    when it is done.'
	"""

	# If blank is given, this means no image
	if not image or image == 'blank':
		image = ""

	# Error check image name
	images = api.image_pool() + [""]
	if image not in images:
		if isatty:
			print '"%s" not a valid image. Please check against output of "show images".' % image
		return 3

	return _createdisk(diskname, size, image, isatty, api)

def _createdisk(diskname, size, image, isatty, api, minsize=1):
	"""Generic create disk function to be called by both 'create disk' and 'clone disk'
	Note that the return values honour both 'create disk' and 'clone disk' documentation."""

	# Check size variable and normalize to be in integer GB
	if not re.match(r'^(\d*\.)?\d+[GgTtMm]$', size):
		if isatty:
			print 'Not a valid size.'
		return 4
	size = size.upper()
	size = int( float(size[:-1]) * 1024.0 ** {'M':-1, 'G':0, 'T':1}[size[-1]] )
	if size < minsize:
		if isatty:
			print 'Size is too small. Disk must be at least %dG, not %dG.' % (minsize, size)
		return 4
	size = str(size) + 'G'

	# check disk doesn't already exist
	disks = api.disk_pool()
	names = [disk['name'] for disk in disks]
	if diskname in names:
		if isatty:
			print 'Cannot create disk "%s": A disk with that name already exists.' % diskname
		return 5

	if image:
		api.deploydisk(diskname=diskname, image=image, size=size)
	else:
		api.createdisk(diskname=diskname, size=size)

	try:
		if isatty:
			sys.stdout.write("Disk creating...")
			sys.stdout.flush()
	
		time.sleep(5)
		diskmade = False
		while not diskmade:
			if isatty:
				sys.stdout.write('.') # Illusion of progress so user doesn't think it's frozen
				sys.stdout.flush()
			disks = api.disk_pool()
			for disk in disks:
				if disk['name'] == diskname and disk['locked'] == False:
					diskmade = True
			time.sleep(5)
	except BaseException, e:
		if isatty:
			print e
			print '''Warning: Disk is still creating. Please check disk is fully created
         and listed in "show disks" before trying to use it.'''
		return 6
	else:
		if isatty:
			print "Created disk %s" % diskname


def clonedisk(diskname, source, size, isatty, api):
	"""call: 'clone disk NAME from SOURCE [with size SIZE]'
	description: 'Create a new disk called NAME, with contents copied from the disk called SOURCE.'
	args: 'NAME: The name of the new disk.
	      .SOURCE: The name of the disk to copy from. Must not be locked.
	      .SIZE: The size of the new disk (see the same arg for command "create disk").
	      .      Must be at least the size of the source disk.
	      .      If not provided, defaults to the size of the source disk.'
	errors:	'If the given source disk does not exist or is busy, exit code 3 is returned.
	        .If the size is too small or otherwise not valid, exit code 4 is returned.
	        .If there already exists a disk with name NAME, exit code 5 is returned.
	        .If an error occurs before creation is complete, exit code 6 is returned.
	        .    NOTE: The creation continues in the background, you must "show disks" to check
	        .    when it is done.'
	"""

	disk_pool = api.disk_pool()
	for disk in disk_pool:
		if disk['name'] == source:
			source_disk = disk
			if disk['locked']:
				if isatty:
					print 'Cannot clone from disk "%s": Disk is locked. Maybe you need to detach it first?' % source
				return 3
			break
	else:
		if isatty:
			print 'Source disk "%s" does not exist.' % source
		return 3

	if not size:
		size = str(source_disk['size'])+'G'

	return _createdisk(diskname, size, source, isatty, api, minsize=source_disk['size'])


def createip(friendly, isatty, api):
	"""call: 'create ip with friendly NAME'
	description: 'Allocate a new IP address and call it NAME.'
	args: 'NAME: The friendly name to assign to the new address.'
	"""
	ip = api.allocateip(friendly=friendly) # If only all the bindings were this simple...

	if isatty:
		print "Ip", ip['ip'], "allocated"
	

def createvm(hostname, ram, type, istty, api):
	"""call: 'create vm NAME with ram SIZE [of type TYPE]'
	description: 'Create a new VM and call it NAME. Give it SIZE amount of memory.
	             .Optionally, a VM Type can be provided. This is for advanced use only.'
	args: 'NAME: The name to give to the new VM.
	      .SIZE: Amount of RAM to allocate to the new VM.
	      .      Format is a number followed by M for megabytes or G for gigabytes.
	      .TYPE: Either "paravirt" or "HVM". Note that not all distros support all types.
	      .      "HVM" should be used when creating Microsoft Windows VMs. "paravirt" should
	      .      otherwise be used and is the default if this argument is not given.'
	errors: 'Incorrect format for SIZE: Return exit code 3
	        .Invalid VM Type: Return exit code 4'
	"""

	ram = ram.upper()
	if not re.match('[1-9][0-9]*(G|M)', ram):
		if istty:
			print 'Bad format for ram size: Must be a number followed by "M" for megabytes or "G" for gigabytes.'
		return 3

	if type:
		if type not in ('paravirt', 'HVM'):
			if istty:
				print 'Invalid type "%s": Must be "paravirt" or "HVM"' % type
			return 4
	else:
		type = 'paravirt'

	vmid = api.allocatevm(hostname=hostname, ram=ram, vm_type=type)

	if vmid > 0:
		if istty:
			print "Successfully created VM %s" % hostname
	else:
		raise Exception("allocatevm returned 0") # This will be caught and "unknown error" presented.



from show import getvmby
from webbindings import HTTPException

def addcontext(vmname, key, value, istty, api):
	"""call: 'add context to vm VM with KEY = VALUE'
	description: 'Add a new context item into the key value context storage for the given VM.
	             .Valid keys must not contain whitespace but values may.
	             .If a key already exists, it will be overwritten.'
	args: 'VM: Name of the vm to attach the key value data to.
	      .KEY: The key to map the given value to. May not contain whitespace.
	      .VALUE: The value to attach to the given key.'
	errors: 'If the VM does not exist: Return exit code 3'
	"""
	key = key.upper()

	vm = getvmby(vmname, api, what='hostname')
	if not vm:
		if istty:
			print 'VM %s does not exist' % vmname
		return 3

	api.set_context(vmid=vm['vm_id'], key=key, value=value)


def clearcontext(key, vmname, istty, api):
	"""call: 'clear context key KEY from vm VM'
	description: 'Remove the given key value data from the given VM context store.
	             .THIS FUNCTION IS NOT YET IMPLEMENTED'
	args: 'KEY: Key of the data to remove from the key value store
	      .VM: The name of the VM to clear the key value data from.'
	errors: 'If the given vm does not exist, exit code 3 is returned
	        .If the given key does not exist in the context store, exit code 4 is returned'
	"""
	key = key.upper()

	vm = getvmby(vmname, api, what='hostname')
	if not vm: 
		if istty:
			print "VM %s does not exist" % vmname
		return 3

	try:
		raise Exception("This function has not been implemented yet. Stay tuned.")
		# api.set_context(vmid=vm['vmid'],key=key,value=None)
	except HTTPException, e:
		if e.retcode != 404:
			raise
		if istty:
			print 'VM %s does not have context key %s' % (vmname, key)
		return 4

	if istty:
		print 'Removed key %s from vm %s' % (key, vmname)



import os
from subprocess import Popen
from tempfile import NamedTemporaryFile as tmpfile
from show import getvmby

# TODO A few things about this need a rewrite.
# Being able to see password on process list, big issue on multi-user systems.
# An attempted (unfinished) rewrite can be found in console-rewrite.py,
# but I think it'll have to wait until the console server behaviour changes.

def runconsole(vmname, istty, api):
	"""call: 'console VM'
	description: 'Log into the out-of-band management console on the given instance.
	             .This is roughly equivilent to directly connecting to the serial port on the machine.
	             .This command requires the programs "grep" and "ssh" be installed
	             .and accessible in a folder listed in the PATH environment variable.
	             .See NOTES section of the man page for extra notes.'
	args: 'VM: The instance to connect to.'
	errors: 'VM does not exist: Return exit code 3
	        .Fail to run ssh: Return exit code 4
	        .ssh exits with non-zero code (generic error): Return exit code 5
	        .ssh exits with code 255 (connection or protocol error): Return exit code 6'
	"""

	console_ip = '49.156.16.12' # IP of OVM console server
	console_rsa = "%s ssh-rsa AAAAB3NzaC1yc2EAAAABIwAAAQEA1ZKuRj8UkoCUvGKGSGd9vQAlP3uCmq+8vBdF/SeZ2sr0Gf9+ZTn5Di8EGBBwOArU791VTtTWTg2kSC6xbearWH9xxD8omnjaYyBmqBLZ0yimVuIQWh3QS5YglxdQoGZUJ7a7ddQDLvO11f4eirP6HcNYSfGT5070jqoEiETmdcoQsdsxdJFs6GBssMoMij1i4HRDbCDPMdViEOQ19LQBCd3LsTFcmZJ/LIO9BCxsSeyV5IPkUVVzVc29JOmqDbCTcHuOidrupVheSSkZjhB0Cq6L8tOaFP/5gj7Ab6PiZPC3hOoLFgPJ3zk50RfAT2/enKqwHQFnN1QzfBBMg1kJiw==" % console_ip
	
	# Check vmname exists. This also checks user/pswd is valid.
	if not getvmby(vmname, api, what='hostname'):
		if istty:
			print 'Cannot find vm "%s"' % vmname
		return 3
	
	# Write a temporary known hosts file, which means we don't clutter up user's ~/.ssh/known_hosts:
	hostfile = tmpfile()
	hostfile.write(console_rsa)
	hostfile.flush()

	try:
		proc = Popen(['ssh', '-qqt', '-oGlobalKnownHostsFile=%s' % hostfile.name, 'console@%s' % console_ip, "%s %s %s" % (api.user, vmname, api.pswd)])
	except OSError:
		if istty:
			print 'Failed to start remote shell. Is the program "ssh" installed and accessbile?'
		return 4

	ret = proc.wait()
	if ret:
		if istty:
			print 'Remote shell failed with exit code %d' % ret
		if ret == 255:
			return 6
		else:
			return 5



#!/usr/bin/env python

from setuptools import setup
from ovm_ctl import __version__

setup(name='ovm_ctl',
    version=__version__,
    license='BSD Simplified 2-clause licence',
    description='Python bindings and command line utility for the OrionVM IaaS platform',
    long_description=open('README').read(),
    url='https://github.com/orionvm/ovm-ctl',
    maintainer='Chris McClymont',
    maintainer_email='chris.mcclymont@orionvm.com',
    packages=['ovm_ctl'],
    scripts=['ovm_ctl/ovm-ctl'],
    package_data={
        'ovm_ctl': [ 'ca.pem', 'extra', 'ovm-ctl' ],
    },
    data_files=[ ('man/man1', ['manpage/ovm-ctl.1']),
		 ('', ['README', 'LICENCE']) ],
    zip_safe=False
)



import random
from context import addcontext


def setpass(vmname, password, istty, api):
	"""call: 'set password on vm VM[ to PASSWORD]'
	description: 'On next boot, vm will have password set to the given password.
	             .Note that this requires we store the password in our system, in plaintext.
	             .As such you may wish to instead use a temporary password here then change it manually.
	             .To this end, a random password is chosen if none is provided.'
	output: '"%s\n"   ->   New password'
	args: 'VM: The vm to set the password of. Note that while a change may be made while the vm is running,
	      .    changes will not take effect until the next boot.
	      .PASSWORD: Optional. The new password to set. May contain whitespace.
	      .    If not given, a random 8 character password is generated.'
	errors: 'If the VM does not exist: Return exit code 3'
	"""
	if not password:
		password = genrandom()
	ret = addcontext(vmname, 'PASSWORD', password, istty, api)
	if ret:
		return ret
	if istty:
		print 'Set password to "%s"' % password
	else:
		print password
	return 0

def genrandom():
	alphabet = [chr(x) for x in range(ord('0'),ord('~'))]
	return ''.join(random.sample(alphabet, 8))



from routes import routes
from parsedocs import docs2dict

def helping(cmd, api, istty):
	"""call: 'help [COMMAND]'
	description: 'Print help on the given command. If no command is given, prints a list of commands.'
	args: 'COMMAND: Optional arg. If given, it specifies a command to print detailed help on.
	      .In general, the name of a command is the part of the command before the first arg, eg: "create vm", "console".'
	errors: 'Command not found: Returns exit code 3'
	"""

	# Note: mostly not affected by istty as its assumed that if the user is asking for help, they want stuff to be output.

	if cmd == None:
		cmd = ''

	if not cmd:
		print
		print 'Command listing:'
		for name, regex, func in routes:
			print docs2dict(func.__doc__)['call']
		print
		print 'For more detailed help, run "help COMMAND"'
		print 'where COMMAND is the first part of a command, eg. "show ips" or "create vm"'
		return 0

	for name, regex, func in routes+[('help',None,helping)]:
		if name == cmd:
			info = docs2dict(func.__doc__)
			print
			print info['call']
			if 'description' in info:
				print
				print info['description']
			if 'args' in info:
				print
				print info['args']
			if 'output' in info:
				print
				print 'In quiet mode, this function will output in the following format:'
				print info['output']
			if 'errors' in info:
				print
				print 'The following errors have special return codes:'
				print info['errors']
			return 0

	if istty:
		print 'No command found: %s' % cmd
	return 3



from show import getvmby
from time import sleep
from sys import stdout

def shutdownvm(vmname, istty, api):
	"""call: 'shutdown vm VM'
	description: 'Stop running the given instance. While an instance is not running, no RAM usage charges are incurred.
	             .Note: Do not attempt to shut down your instance using the standard shut down feature of its operating system.
	             .Your instance will be restarted immediately and usage charges will continue.'
	args: 'VM: Name of the instance to stop running.'
	errors: 'VM name is invalid or VM does not exist: Return exit code 3
	        .VM not in a state to be shut down (eg. not running): Return exit code 4
	        .An error occurred while waiting for VM to shut down: Return exit code 5
	        .    Note: In this case, the VM may have not shut down correctly.
	        .    You should check with "show vm" and the out-of-band console ("console")
	        .    If the VM remains uncontactable and is still running after several minutes,
	        .    please contact OrionVM technical support.
	        .VM failed to shut down: Return exit code 6
	        .    This is a serious error. Plesae contact OrionVM technical support
	        .    and we will resolve the issue.'
	"""
	vm = getvmby(vmname, api, what='hostname')
	if vm is None:
		if istty:
			print vmname, "does not exist"
		return 3
	vmid = vm['vm_id']
	if vm['state'] != 2:
		if istty:
			print vmname, "is not in state to be shut down"
		return 4

	api.actionvm(vmid=vmid,action='shutdown')

	try:
		if istty:
			stdout.write("Shutting down...")
			stdout.flush()
		while 1:
			sleep(3)
			if istty:
				stdout.write('.')
				stdout.flush()
			vm = getvmby(vmname, api, what='hostname')
			if vm['state'] == 0:
				break
			if vm['state'] > 9:
				if istty:
					print "VM failed to shut down. Please contact OrionVM technical support and we will resolve the issue."
				return 6
	except:
		if istty:
			print "Error occurred while shutting down VM."
			print "Warning: VM may or may not have shut down correctly."
			print '         Please try "show vm %s" to check if VM has shut down.' % vmname
			print '         If VM has not shut down after several minutes'
			print '         and you cannot connect to the out of band console ("console %s"),' % vmname
			print '         we reccomend you contact OrionVM technical support'
			print '         and we will address the situation.'
		return 5

	if istty:
		print "Done"
	



from show import getvmby
from time import sleep
from sys import stdout

def bootvm(vmname, istty, api):
	"""call: 'boot vm VM'
	description: 'Begin running the given instance.'
	args: 'VM: Name of the instance to boot up.'
	errors: 'If VM name is invalid or VM does not exist: Returns exit code 3
	        .If VM is not ready to be booted (eg. is already running): Returns exit code 4
	        .If VM fails to correctly boot (booting is aborted): Returns exit code 5
	        .If an error occurs while booting: Returns exit code 6
	        .    NOTE: In this case, the VM may still boot correctly.
	        .    Try "show vm" to check if the VM is now running or still booting.'
"""
	vm = getvmby(vmname, api, what='hostname')
	if vm is None:
		if istty:
			print vmname, "does not exist"
		return 3
	vmid = vm['vm_id']
	if vm['state'] != 0:
		if istty:
			print vmname, "is not in state to be booted"
		return 4

	api.deploy(vmid=vmid)

	if istty:
		stdout.write('Booting.')
		stdout.flush()
	try:
		while 1:
			sleep(3)
			if istty:
				stdout.write('.') # Give the user some feedback that it isn't frozen
				stdout.flush()
			vm = getvmby(vmname, api, what='hostname')
			if vm['state'] not in (1,2):
				if istty:
					print "Failed to boot"
				return 5
			if vm['state'] == 2:
				break
	except:
		if istty:
			print 'Unknown error while waiting for VM to boot.'
			print 'Warning: VM is not booted. However, it is still booting.'
			print '         Please use "show vm %s" to check if VM successfully booted.' % vmname
		return 6

	if istty:
		print "Done"




import re
from webbindings import HTTPException

# Function to nicely format output into table columns if istty, otherwise tab separated
def printData(header, rows, istty):
    if header == None or not istty:
        allrows = rows
        if len(rows) == 0:
            return
    else:
        allrows = [header]
        allrows.extend(rows)
    
    width = len(allrows[0])
    
    if istty:
        # Find maximum width for each column among all rows
        padding = [0] * width
        for row in allrows:
            for x in range(len(row)):
            	if not isinstance(row[x], str):
            		row[x] = str(row[x])
                length = len(row[x])
                if length > padding[x]:
                    padding[x] = length
    
    # Tab separated for programmatic uses, nicely spaced for ttys
    for row in allrows:
        if istty:
            line = ""
            for x in range(width):
                line += row[x].ljust(padding[x] + 3)
            print line
        else:
            print "\t".join(map(str, row))

def showuse(istty, api): 
	r"""call: 'show usage'
	description: 'Report amount of RAM and HDD Storage being used,
		and the number of IP addresses currently allocated.'
	output: '"%d\t%d\t%d\n" ==> RAM in Megabytes, Storage in Gigabytes, Number of IPs'
	"""
	use = api.usage()
	if istty:
		if use['ram'] > 8*1024:
			use['ram'] /= 1024.0
			use['ram'] = ('%.2f' % use['ram']) + 'G'
		else:
			use['ram'] = str(use['ram']) + 'M'
		print "You are using %(ram)s of ram, %(disk)sG of disk, %(ip)s ips" % use
	else:
		print "%d\t%d\t%d" % (use['ram'], use['disk'], use['ip'])
	return 0

def showbalance(istty, api):
	r"""call: 'show balance'
	description: 'Report account balance, current to the last hour.'
	output: '"%.2f\n" ==> Balance in dollars (to the cent)'
	"""
	bal = api.details()['balance']
	if istty:
		print "Your account balance is $%.2f" % bal
	else:
		print "%.2f" % bal
	return 0

def showips(istty, api):
	r"""call: 'show ips'
	description: 'Print a list of IP addresses and their friendly names.'
	output: '"%s\t%s\n    ==>    IP Address, Friendly Name,
		. %s\t%s\n           IP Address, Friendly Name,
		. ..."               etc...(one per line)'
	"""
	ips = api.ip_pool()
	header = ['IP Address', 'Friendly Name']
	data = [[ip['ip'], ip['friendly']] for ip in ips]
	printData(header, data, istty)


def getvmby(value, api, what='vm_id'):
	"""Return the vm dict of the vm described in the arguments.
	The argument 'what' tells it what field to search on (default vmid).
	The argument 'value' is the expected value of the 'what' field you are searching for."""
	vms = api.vm_pool()
	for vm in vms:
		if vm[what] == value:
			return vm
	return None

def showip(matchip, istty, api):
	r"""call: 'show ip IP'
	description: 'Get information on a specific IP address, including up/down data totals,
		and (if in use) the vm using it.'
	args: 'IP: The IP address or friendly name to get information on.'
	output: '"%s\t%s\t%d\t%d\t%s\n" ==> IP Address, Friendly name,
		.                           Total Uploaded (bytes), Total Downloaded (bytes),
		.                           VM Name of VM using this IP (ommitted if not in use)'
	errors: 'For non-existing IP address or friendly name: Returns exit code 3'
	"""

	ips = api.ip_pool()
	for ip in ips:
		if ip['friendly'] == matchip or ip['ip'] == matchip:
			up = ip['up']
			down = ip['down']
			if istty:
				up, down = [x/float(1024**3) for x in (up, down)] # Convert to GB

			if ip['locked']:				
				vmname = getvmby(ip['vmid'], api, what='vm_id')['hostname']
				data = {'ip': ip['ip'], 'friendly': ip['friendly'], 'up': up, 'down': down, 'vmname': vmname}

				if istty:
					print "%(ip)s aka %(friendly)s %(up).2fG up/%(down).2fG down locked by %(vmname)s." % data
				else:
					print "%(ip)s\t%(friendly)s\t%(up)d\t%(down)d\t%(vmname)s" % data
			else:
				data = {'ip': ip['ip'], 'friendly': ip['friendly'], 'up': up, 'down': down}

				if istty:
					print "%(ip)s aka %(friendly)s %(up).2fGup/%(down).2fGdown is unlocked" % data
				else:
					print "%(ip)s\t%(friendly)s\t%(up)d\t%(down)d" % data
			break
	else:
		if istty:
			print "Cannot find ip", matchip
		return 3
	return 0

def showdisks(istty, api):
	r"""call: 'show disks'
	description: 'Print list of disks, their template image and their size.'
	output: '"%s\t%s\t%d\n    ==>    Disk Name, Image Name, Size (in GB)
		. %s\t%s\t%d\n           Disk Name, Image Name, Size (in GB)
		. ..."                   etc...(one per line)'
	"""
	disks = api.disk_pool()
	for disk in disks:
		if 'image' not in disk or not disk['image']:
			disk['image'] = '[blank]'
		disk['size'] = str(disk['size']) + ("G" if istty else "")

	header = ['Name', 'Image', 'Size']
	data = [  [ d['name'], d['image'], d['size'] ] for d in disks]
	printData(header, data, istty)

	return 0

def showdisk(diskname, istty, api):
	r"""call: 'show disk NAME'
	description: 'Get information on a specific disk, including image name, size and
	              whether the disk is locked (normally, attached to a vm)'
	args: 'NAME: The name of the disk to get information on.'
	output: '"%s\t%s\t%d\t%s\t%s\n" ==> Disk Name, Image Name, Size (in GB),
	        .                           "locked" if disk is locked, else "unlocked",
	        .                           licence if disk a licenced windows disk, else omitted'
	errors: 'Disk does not exist: Returns exit code 3'
	"""
	disks = api.disk_pool()
	for disk in disks:
		if disk['name'] == diskname:
			if 'image' not in disk or not disk['image']:
				disk['image'] = '[blank]'
			if istty:
				s = "%(name)s is of image %(image)s and size %(size)dG and %(islocked)s locked" % \
				{'name': disk['name'], 'image': disk['image'], 'size': disk['size'], 'islocked': ['is not', 'is'][disk['locked']]}
				if 'licence' in disk and disk['licence']:
					s += ' with licence %s' % disk['licence']
				print s
			else:
				print '\t'.join([disk['name'], disk['image'], str(disk['size'])] + (disk['locked'] and ['locked'] or ['unlocked']) + ('licence' in disk and disk['licence'] or []))
			break
	else:
		if istty:
			print 'Disk not found'
		return 3
	return 0
		
# Hi there. Reading through the source? Good for you!

def showvms(istty, api):
	r"""call: 'show vms'
	description: 'Print list of all vms, both running and shutdown.'
	output: '"%s\t%s\t%d\t%d\t%d\t%s\n    ==>    VM Name, VM Type, Memory (in MB), Number of disks, Number of ips, VM State
	        . %s\t%s\t%d\t%d\t%d\t%s\n           VM Name, VM Type, Memory (in MB), Number of disks, Number of ips, VM State
	        . ..."                               etc...(one per line)'
	"""
	vms = api.vm_pool()
	vmstates = {0: 'Shut down', 1: 'Booting', 2: 'Running', 3: 'Shutting down'}

	for vm in vms:
		if istty:
			if vm['ram'] >= 2048:
				vm['ram'] = "%.2fG" % (vm['ram'] / 1024.0)
			else:
				vm['ram'] = "%dM" % vm['ram']
		vm['state'] = vmstates.get(vm['state'], 'Erroring')

	
	header = ['Name', 'Type', 'Memory', 'Disks', 'IPs', 'State']
	data = [  [vm['hostname'], vm['vm_type'], vm['ram'], 
			  len(vm['disks']), len(vm['ips']), vm['state'] ] for vm in vms]
	
	printData(header, data, istty)
	
	return 0

def showvm(vmhost, istty, api):
	r"""call: 'show vm NAME'
	description: 'Get information on a specific VM, including running state,
	              attached disks and attached ips.'
	args: 'NAME: The name of the VM to get information on.'
	output: '"%s\t%s\t%d\t%s\n    ==>    VM Name, Type, Memory (in MB), State
	        . disk\t%s\t%s\t%s\n         Disk Name, Template, Device Name on VM
	        . disk\t%s\t%s\t%s\n         Disk Name, Template, Device Name on VM
	        . ...                        etc (one disk per line)
	        . ip\t%s\n                   IP Address
	        . ip\t%s\n                   IP Address
	        . ..."                       etc (one ip per line)'
	errors: 'For non-existing VM Name: Return exit code 3'
	"""
	# TODO when possible: Output list of context items
	vm = getvmby(vmhost, api, what='hostname')
	if vm is None:
		print "Cannot find vm", vmhost
		return 3

	vmstates = {0: 'Shut down', 1: 'Booting', 2: 'Running', 3: 'Shutting down'}
	if istty:
		if vm['ram'] >= 2048:
			ram = "%.2fG" % (vm['ram'] / 1024.0)
		else:
			ram = "%dM" % vm['ram']
		# Awful line. Wish print didn't behave so stupid. Wish python had a ?: ternary operator.
		print '%s of type %s has size %s and is currently %s' % (vm['hostname'], vm['vm_type'], ram, vmstates.get(vm['state'], "Erroring"))
	else:
		print '%s\t%s\t%d\t%s' % (vm['hostname'], vm['vm_type'], vm['ram'], vmstates.get(vm['state'], "Erroring"))
	for disk in vm['disks']:
		if 'image' not in disk or not disk['image']:
			disk['image'] = '[blank]'
		if istty:
			print "Contains disk %(name)s of image %(image)s as device %(target)s" % disk
		else:
			print "disk\t%(name)s\t%(image)s\t%(target)s" % disk
	
	if istty and vm['disks'] == []:
		print "Contains no disks"
	
	for ip in vm['ips']:
		print (istty and "Contains ip " or "ip\t") + ip

	if istty and vm['ips'] == []:
		print "Contains no ips"

	return 0

def get_ip(ip_str, api, istty):
	"""Convert a string into an ip address string. If already an address, no change. If is a valid friendly, replaces with the friendly's address. If doesn't match any friendly and not a valid address, returns ''"""
	
	ip_pool = api.ip_pool()
	for ip in ip_pool:
		if ip['friendly'] == ip_str:
			return ip['ip']
	parts = ip_str.split('.')
	if len(parts) != 4:
		return ''
	for part in parts:
		if not (0 <= int(part) < 256 and str(int(part)) == part):
			return ''
	return ip_str # is a valid ip

def showimages(istty, api):
	r"""call: 'show images'
	description: 'Get a list of available software distributions for creating new instances.
	             .Please note that the "blank" value is special, and is used to create a completely empty disk.'
	output: '"%s\n    ==>    Image
	        . %s\n           Image
	        . ..."           etc (one per line)'
	"""
	
	images = api.image_pool()

	for image in images:
		print image

def showcontext(key, vmname, istty, api):
	r"""call: 'show context key KEY on vm VM'
	description: 'Retrieve the value for the given key from the key-value context information associated with the given vm.'
	args: 'KEY: The context key to look up in the vm context store
	      .VM: The name of the vm to look up context information on'
	output: '"%s\n"    ==>    Value'
	errors: 'If the given vm does not exist, exit code 3 is returned.
	        .If the given vm context store does not contain the given key, exit code 4 is returned.'
	"""

	vm = getvmby(vmname, api, what='hostname')
	if not vm:
		if istty:
			print "VM %s does not exist" % vmname
		return 3

	try:
		value = api.get_context(vmid=vm['vm_id'],key=key)
	except HTTPException, e:
		if e.retcode != 404:
			raise
		if istty:
			print 'VM %s does not have context key %s' % (vmname, key)
		return 4

	if istty:
		print 'For vm %s:\t%s = %s' % (vmname, key, value)
	else:
		print value



from show import getvmby, get_ip

def detachdisk(diskname, vmname, istty, api):
	"""call: 'detach disk DISK from VM'
	description: 'Detach the given disk from the instance its attached to, removing it from that system but freeing it to be destroyed or attached elsewhere.'
	args: 'DISK: Name of the disk to detach.
	      .VM: Name of the instance that the disk is currently attached to.'
	errors: 'VM does not exist or is not shut down: Returns exit code 3
	        .Disk not attached to VM, or disk does not exist: Returns exit code 4'
	"""

	vm = getvmby(vmname, api, what='hostname')
	if vm is None:
		if istty:
			print "VM %s does not exist" % vmname
		return 3
	if vm['state'] != 0:
		if istty:
			print "VM %s is not shut down" % vmname
		return 3

	found = False
	for disk in vm['disks']:
		if disk['name'] == diskname:
			found = True
	if not found:
		if istty:
			print "Disk %s not attached to VM %s" % (diskname, vmname)
		return 4

	vmid = vm['vm_id']
	api.detachdisk(vmid=vmid, diskname=diskname)

	if istty:
		print "Detached %s from %s" % (diskname, vmname)

def detachip(ipname, vmname, istty, api):
	"""call: 'detach ip IP from VM'
	description: 'Detach the given IP from the instance its attached to, allowing it to be freed or attached to a different instance instead.'
	args: 'IP: The IP address or friendly name you want to detach.
	      .VM: The name of the instance that the ip is currently attached to.'
	errors: 'VM does not exist: Returns exit code 3
	        .Invalid IP or no matching friendly IP name found: Returns exit code 4'
	"""
	vm = getvmby(vmname, api, what='hostname')
	if vm is None:
		if istty:
			print "VM %s does not exist" % vmname
		return 3

	# look up ip by friendly name
	ip = get_ip(ipname, api, istty)
	if not ip:
		if istty:
			print 'IP address invalid, or no matching friendly name found.'
		return 4
	
	vmid = vm['vm_id']
	api.detachip(ip=ip, vmid=vmid)

	if istty:
		print "Detached %s from %s" % (ipname, vmname)
